/********************************************************************************
 * Copyright (c) 2021-2023 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

#pragma once

#include <MantleAPI/Execution/i_environment.h>
#include <agnostic_behavior_tree/action_node.h>

#include <cassert>
#include <utility>

#include "Storyboard/ByEntityCondition/ReachPositionCondition_impl.h"
#include "Utils/EntityBroker.h"

namespace OpenScenarioEngine::v1_2::Node
{
class ReachPositionCondition : public yase::ActionNode
{
public:
  ReachPositionCondition(std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_2::IReachPositionCondition> reachPositionCondition)
      : yase::ActionNode{"ReachPositionCondition"},
        reachPositionCondition_{reachPositionCondition}
  {
  }

  void onInit() override{};

private:
  yase::NodeStatus tick() override
  {
    assert(entityBroker_);
    assert(impl_);

    const auto is_satisfied = impl_->IsSatisfied();

    if (is_satisfied)
    {
      entityBroker_->add_triggering(triggeringEntity_);
      return yase::NodeStatus::kSuccess;
    }

    return yase::NodeStatus::kRunning;
  };

  void lookupAndRegisterData(yase::Blackboard& blackboard) final
  {
    triggeringEntity_ = blackboard.get<std::string>("TriggeringEntity");
    entityBroker_ = blackboard.get<EntityBroker::Ptr>("EntityBroker");

    auto environment = blackboard.get<std::shared_ptr<mantle_api::IEnvironment>>("Environment");

    impl_ = std::make_unique<OpenScenarioEngine::v1_2::ReachPositionCondition>(
        OpenScenarioEngine::v1_2::ReachPositionCondition::Values{
            triggeringEntity_,
            reachPositionCondition_->GetTolerance(),
            [=]() { return ConvertScenarioPosition(environment, reachPositionCondition_->GetPosition()); }},
        OpenScenarioEngine::v1_2::ReachPositionCondition::Interfaces{
            environment});
  }

  std::unique_ptr<OpenScenarioEngine::v1_2::ReachPositionCondition> impl_{nullptr};
  std::string triggeringEntity_;
  EntityBroker::Ptr entityBroker_{nullptr};
  std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_2::IReachPositionCondition> reachPositionCondition_;
};

}  // namespace OpenScenarioEngine::v1_2::Node