
/********************************************************************************
 * Copyright (c) 2021-2023 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 ********************************************************************************/

#include "Conversion/OscToMantle/ConvertScenarioTimeToCollisionConditionTarget.h"

#include "Conversion/OscToMantle/ConvertScenarioEntity.h"
#include "Conversion/OscToMantle/ConvertScenarioPosition.h"

namespace OpenScenarioEngine::v1_2
{
TimeToCollisionConditionTarget ConvertScenarioTimeToCollisionConditionTarget(
    const std::shared_ptr<mantle_api::IEnvironment>& environment,
    const std::shared_ptr<NET_ASAM_OPENSCENARIO::v1_2::ITimeToCollisionConditionTarget>& timeToCollisionConditionTarget)
{
  if (auto position = timeToCollisionConditionTarget->GetPosition(); position)
  {
    return ConvertScenarioPosition(environment, position);
  }
  if (auto entity = timeToCollisionConditionTarget->GetEntityRef(); entity)
  {
    return ConvertScenarioEntity(entity);
  }
  throw std::runtime_error("Unable to convert TimeToCollisionConditionTarget: Either Position or EntityRef must be set\n");
}

}  // namespace OpenScenarioEngine::v1_2